; ---------------------------------------------------
; 2x2 Printer Routine
; (c) 2017-2018 Ast A. Moore
; Adapted for the Woot! 2018 magazine
;
; (Best viewed with tab size set to 8 characters)
;
; Visit my website at http://relative-path.com/zx/
; for more Spectrum-related materials
; ---------------------------------------------------


		ld de,$58e0		;load DE with screen attribute address
		call printer
		defm "Merry Christmas!",0

		ret


printer
;Fetch character bitmap address:

		pop hl			;grab stack address (after CALL; beginning of string)
		ld a,(hl)		;value at address into A for testing
		inc hl			;next address
		push hl			;back onto the stack
		or a
		ret z			;RETurn to the address right after EOS
		
		push de			;save current screen attr address

		ld h,0			;make sure H=0
		add a,a			;			\
		ld l,a			;load L with chr code
		add hl,hl		; 			  multiply HL by 8
		add hl,hl		;			/
		ld bc,$3c00		;add $3c00 to get the base address of character
		add hl,bc		;bitmap in ROM - $100 (chr code*8+$3c00), and load it into HL

		ex de,hl		;bitmap addr into DE, screen addr into HL

		push de			;save bitmap addr onto stack
		push hl			;save screen addr onto stack

		ld a,4			;skip nibble swap (for the self-modifying code)
		
		call print_half		;print left half

		pop hl			;restore screen addr
		inc l			;move one cell to the right

		pop de			;restore bitmap addr

		xor a			;don't skip nibble swap (for the self-modifying code)

		call print_half		;now print right half
		
		pop de			;restore screen address
		inc e			;move two cells to the right
		inc e
		
		jr printer


print_half
		ld (sw+1),a		;inject A (displacement) into the JR instruction below

		call pr_hlf
		
		ld bc,32		;add 32 to screen addr
		add hl,bc		;to move 8 px down

pr_hlf		push hl			;save screen attr addr onto stack

		ld a,h			;load H into A
		rla			;to convert attr address into bitmap address
		rla
		rla
		and 88
		ld h,a			;back into H

		ld b,4			;half the number of lines, as each one will be doubled
bitmap		ld a,(de)		;bitmap into A: abcdxxxx
sw		jr $+1			;SMC
		add a,a			;shift low-order nibble
		add a,a			;into high-order nibble
		add a,a			;(only for the right half of character)
		add a,a

		push hl			;save HL, since we'll be destroying it

		sra a			;shift right, retaining bit 7: aabcdxxx
		ld c,a			;copy into C
		and %11100000		;isolate bits 7–5: aab00000
		ld h,a			;save in H

		ld a,c			;aabcdxxx
		rra			;0aabcdxx
		ld c,a			;back into C
		and %00011000		;000bc000
		or h			;merge with H: aabbc000
		ld h,a			;save back in H

		ld a,c			;0aabcdxx
		rra			;00aabcdx
		and %00000110		;00000cd0
		or h			;aabbccd0
		ld h,a

		rra			;0aabbccd
		and 1			;0000000d
		or h			;aabbccdd

		pop hl			;restore HL

		ld (hl),a		;put on screen
		inc h			;one line down
		ld (hl),a		;duplicate
		inc h			;one line down
		inc e			;next bitmap addr
		djnz bitmap

		pop hl			;restore screen attr addr

		ret